/*
***************************************************************************
*
* Author: Teunis van Beelen
*
* Copyright (C) 2010 Teunis van Beelen
*
* teuniz@gmail.com
*
***************************************************************************
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation version 2 of the License.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
***************************************************************************
*
* This version of GPL is at http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
*
***************************************************************************
*/


/*
*****************************************************************************
*
* Spectraalanalyze via "SFT" (Straight/Slow Fourier Transform)
*
* Tijdsignaal X
* Gesampled naar N samples in X0, X1, X2, ... Xn-1
* Sample interval t(tau) seconden (b.v. 0.01 bij 100 Hz samplefreq.)
*
* Powerspectrum P
* Berekend voor N/2 frequentiepunten op P0, P1, P2, ... Pn/2-1
* Frequentieresolutie dF(delta F) = 1/Nt (b.v. 0.1 bij 10 sec. signaal (N = 1000))
*
* Berekening Pi, dus voor de frequentie i * dF (i = 0, 1, 2, ... N/2-1)
*
* REi = t * {X0 * cos((2*pi*i/N*t) 0 * t)  +  X1 * cos((2*pi*i/N*t) 1 * t)  +  X2 * cos((2*pi*i/N*t) 2 * t)  +  ...  +  XN-1 * cos((2*pi*i/N*t) (N-1) * t)}
*
* IMi = t * {X0 * sin((2*pi*i/N*t) 0 * t)  +  X1 * sin((2*pi*i/N*t) 1 * t)  +  X2 * sin((2*pi*i/N*t) 2 * t)  +  ...  +  XN-1 * sin((2*pi*i/N*t) (N-1) * t)}
*
* Pi = (2/(N*t*N*t)) * {REi*REi + IMi*IMi}
*
*****************************************************************************
*/



#include "fourier.h"


#define _GNU_SOURCE


#define DUB_PI 6.28318530717958623199592693708837032318115234375



float fourier_transform_get_power_float(int freq, float *buf, int bufsize, float samplefreq)
{
  int i;

  float tau,
        deltafreq,
        rei,
        imi,
        itau,
        sizetau,
        dub_pi_freq,
        var_x,
        result;

  dub_pi_freq = DUB_PI * (float)freq;

  tau = 1.0 / samplefreq;

  sizetau = tau * bufsize;

  deltafreq = 1.0 / sizetau;

  var_x = dub_pi_freq / sizetau;

  rei = 0.0;

  imi = 0.0;

  for(i=0; i<bufsize; i++)
  {
    itau = tau * i;

    rei += (tau * (buf[i] * cosf(var_x * itau)));

    imi += (tau * (buf[i] * sinf(var_x * itau)));
  }

  result = (2.0 / (sizetau * sizetau)) * ((rei * rei) + (imi * imi));

  if(!freq)
  {
    return(result / 2.0);
  }

  return(result);
}


double fourier_transform_get_power_double(int freq, double *buf, int bufsize, double samplefreq)
{
  int i;

  double tau,
         deltafreq,
         rei,
         imi,
         itau,
         sizetau,
         dub_pi_freq,
         var_x,
         result;

  dub_pi_freq = DUB_PI * (double)freq;

  tau = 1.0 / samplefreq;

  sizetau = tau * bufsize;

  deltafreq = 1.0 / sizetau;

  var_x = dub_pi_freq / sizetau;

  rei = 0.0;

  imi = 0.0;

  for(i=0; i<bufsize; i++)
  {
    itau = tau * i;

    rei += (tau * (buf[i] * cos(var_x * itau)));

    imi += (tau * (buf[i] * sin(var_x * itau)));
  }

  result = (2.0 / (sizetau * sizetau)) * ((rei * rei) + (imi * imi));

  if(!freq)
  {
    return(result / 2.0);
  }

  return(result);
}













